/**CFile****************************************************************

  FileName    [ntk.h]

  PackageName [MVSIS 2.0: Multi-valued logic synthesis system.]

  Synopsis    [Declarations of the network/node package.]

  Author      [MVSIS Group]
  
  Affiliation [UC Berkeley]

  Date        [Ver. 1.0. Started - February 1, 2003.]

  Revision    [$Id: ntk.h,v 1.45 2003/05/27 23:14:20 alanmi Exp $]

***********************************************************************/
 
#ifndef __NTK_H__
#define __NTK_H__


////////////////////////////////////////////////////////////////////////
///                          INCLUDES                                ///
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
///                         PARAMETERS                               ///
////////////////////////////////////////////////////////////////////////

// node types: no type, primary input, primary output, internal
typedef enum { MV_NODE_NONE, MV_NODE_CI, MV_NODE_CO, MV_NODE_INT }  Ntk_NodeType_t;
// node subtype for combinational inputs/outputs: 
// node is a PI/PO of the network; node is only an input/output a latch
typedef enum { MV_BELONGS_TO_NET, MV_BELONGS_TO_LATCH }  Ntk_NodeSubtype_t;

////////////////////////////////////////////////////////////////////////
///                         BASIC TYPES                              ///
////////////////////////////////////////////////////////////////////////

typedef struct NtkPinStruct        Ntk_Pin_t;       // fanin/fanout pin (16 bytes)
typedef struct NtkNodeStruct       Ntk_Node_t;      // the node data structure
typedef struct NtkLatchStruct      Ntk_Latch_t;     // the latch data structure
typedef struct NtkNetworkStruct    Ntk_Network_t;   // the network data structure
typedef struct NtkNodeHeapStruct   Ntk_NodeHeap_t;  // the heap of nodes

////////////////////////////////////////////////////////////////////////
///                     FUNCTION DEFITIONS                           ///
////////////////////////////////////////////////////////////////////////

/*=== ntk.c ==========================================================*/
extern void              Ntk_Init( Mv_Frame_t * pMvsis );
extern void              Ntk_End( Mv_Frame_t * pMvsis );
/*=== ntkApi.c =======================================================*/
// "ntkApi.c" contains only simple lookup/insert type of APIs
// more complex APIs are located in other files detailed below
// data-access pin APIs
extern Ntk_Pin_t *       Ntk_PinReadNext( Ntk_Pin_t * pPin );
extern Ntk_Pin_t *       Ntk_PinReadPrev( Ntk_Pin_t * pPin );
extern Ntk_Pin_t *       Ntk_PinReadLink( Ntk_Pin_t * pPin );
extern Ntk_Node_t *      Ntk_PinReadNode( Ntk_Pin_t * pPin );
// data-access latch APIs
extern Ntk_Latch_t *     Ntk_LatchReadNext( Ntk_Latch_t * pLatch );
extern Ntk_Latch_t *     Ntk_LatchReadPrev( Ntk_Latch_t * pLatch );
extern int               Ntk_LatchReadReset( Ntk_Latch_t * pLatch );
extern Ntk_Node_t *      Ntk_LatchReadInput( Ntk_Latch_t * pLatch );
extern Ntk_Node_t *      Ntk_LatchReadOutput( Ntk_Latch_t * pLatch );
extern Ntk_Node_t *      Ntk_LatchReadNode( Ntk_Latch_t * pLatch );
extern Ntk_Network_t *   Ntk_LatchReadNet( Ntk_Latch_t * pLatch );
extern char *            Ntk_LatchReadData( Ntk_Latch_t * pLatch );
// data-access node APIs
extern Ntk_Node_t *      Ntk_NodeReadNext( Ntk_Node_t * pNode );
extern Ntk_Node_t *      Ntk_NodeReadPrev( Ntk_Node_t * pNode );
extern Ntk_Node_t *      Ntk_NodeReadOrder( Ntk_Node_t * pNode );
extern int               Ntk_NodeReadId( Ntk_Node_t * pNode );
extern char *            Ntk_NodeReadName( Ntk_Node_t * pNode );
extern Ntk_NodeType_t    Ntk_NodeReadType( Ntk_Node_t * pNode );
extern Ntk_NodeSubtype_t Ntk_NodeReadSubtype( Ntk_Node_t * pNode );
extern int               Ntk_NodeReadValueNum( Ntk_Node_t * pNode );
extern Ntk_Pin_t *       Ntk_NodeReadFaninPinHead( Ntk_Node_t * pNode );
extern Ntk_Pin_t *       Ntk_NodeReadFaninPinTail( Ntk_Node_t * pNode );
extern int               Ntk_NodeReadFaninNum( Ntk_Node_t * pNode );
extern Ntk_Pin_t *       Ntk_NodeReadFanoutPinHead( Ntk_Node_t * pNode );
extern Ntk_Pin_t *       Ntk_NodeReadFanoutPinTail( Ntk_Node_t * pNode );
extern int               Ntk_NodeReadFanoutNum( Ntk_Node_t * pNode );
extern char *            Ntk_NodeReadData( Ntk_Node_t * pNode );
extern Ntk_Network_t *   Ntk_NodeReadNetwork( Ntk_Node_t * pNode );
extern Fnc_Manager_t *   Ntk_NodeReadMan( Ntk_Node_t * pNode );
extern Fnc_Function_t *  Ntk_NodeReadFunc( Ntk_Node_t * pNode );
extern FILE *            Ntk_NodeReadMvsisErr( Ntk_Node_t * pNode );
extern FILE *            Ntk_NodeReadMvsisOut( Ntk_Node_t * pNode );
// node functionality APIs
extern Vm_VarMap_t *     Ntk_NodeReadFuncVm( Ntk_Node_t * pNode );
extern Cvr_Cover_t *     Ntk_NodeReadFuncCvr( Ntk_Node_t * pNode );
extern Mvr_Relation_t *  Ntk_NodeReadFuncMvr( Ntk_Node_t * pNode );
extern DdNode **         Ntk_NodeReadFuncGlo( Ntk_Node_t * pNode );
extern Vm_VarMap_t *     Ntk_NodeGetFuncVm( Ntk_Node_t * pNode );
extern Cvr_Cover_t *     Ntk_NodeGetFuncCvr( Ntk_Node_t * pNode );
extern Mvr_Relation_t *  Ntk_NodeGetFuncMvr( Ntk_Node_t * pNode );
extern DdNode **         Ntk_NodeGetFuncGlo( Ntk_Node_t * pNode );
extern void              Ntk_NodeWriteFuncVm( Ntk_Node_t * pNode, Vm_VarMap_t * pVm );
extern void              Ntk_NodeWriteFuncCvr( Ntk_Node_t * pNode, Cvr_Cover_t * pCvr );
extern void              Ntk_NodeWriteFuncMvr( Ntk_Node_t * pNode, Mvr_Relation_t * pMvr );
extern void              Ntk_NodeWriteFuncGlo( Ntk_Node_t * pNode, DdNode ** pGlo );
extern void              Ntk_NodeSetFuncVm( Ntk_Node_t * pNode, Vm_VarMap_t * pVm );
extern void              Ntk_NodeSetFuncCvr( Ntk_Node_t * pNode, Cvr_Cover_t * pCvr );
extern void              Ntk_NodeSetFuncMvr( Ntk_Node_t * pNode, Mvr_Relation_t * pMvr );
extern void              Ntk_NodeSetFuncGlo( Ntk_Node_t * pNode, DdNode ** pGlo );
extern void              Ntk_NodeFreeFuncVm( Ntk_Node_t * pNode );
extern void              Ntk_NodeFreeFuncCvr( Ntk_Node_t * pNode );
extern void              Ntk_NodeFreeFuncMvr( Ntk_Node_t * pNode );
extern void              Ntk_NodeFreeFuncGlo( Ntk_Node_t * pNode );
extern int               Ntk_NodeReadDefault( Ntk_Node_t * pNode );
extern Fnc_Manager_t *   Ntk_NodeReadMan   ( Ntk_Node_t * pNode );
extern Mvr_Manager_t *   Ntk_NodeReadManMvr( Ntk_Node_t * pNode );
extern Vm_Manager_t *    Ntk_NodeReadManVm ( Ntk_Node_t * pNode );
extern Vmx_Manager_t *   Ntk_NodeReadManVmx( Ntk_Node_t * pNode );
extern Mvc_Manager_t *   Ntk_NodeReadManMvc( Ntk_Node_t * pNode );
// data-access network APIs
extern char *            Ntk_NetworkReadName( Ntk_Network_t * pNet );
extern char *            Ntk_NetworkReadSpec( Ntk_Network_t * pNet );
extern Mv_Frame_t *      Ntk_NetworkReadMvsis( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkReadCiHead( Ntk_Network_t * pNet ); // pi + latch outputs
extern Ntk_Node_t *      Ntk_NetworkReadCiTail( Ntk_Network_t * pNet );
extern int               Ntk_NetworkReadCiNum( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkReadCoHead( Ntk_Network_t * pNet ); // po + latch inputs
extern Ntk_Node_t *      Ntk_NetworkReadCoTail( Ntk_Network_t * pNet );
extern int               Ntk_NetworkReadCoNum( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkReadNodeHead( Ntk_Network_t * pNet ); // only intenal
extern Ntk_Node_t *      Ntk_NetworkReadNodeTail( Ntk_Network_t * pNet );
extern int               Ntk_NetworkReadNodeIntNum( Ntk_Network_t * pNet );
extern Ntk_Latch_t *     Ntk_NetworkReadLatchHead( Ntk_Network_t * pNet ); // latches
extern Ntk_Latch_t *     Ntk_NetworkReadLatchTail( Ntk_Network_t * pNet );
extern int               Ntk_NetworkReadLatchNum( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkReadOrder( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkReadOrderByLevel( Ntk_Network_t * pNet, int Level );
extern int               Ntk_NetworkReadNodeTotalNum( Ntk_Network_t * pNet ); // sum total of all nodes (w/o latches)
extern int               Ntk_NetworkReadNodeWritableNum( Ntk_Network_t * pNet );
extern FILE *            Ntk_NetworkReadMvsisErr( Ntk_Network_t * pNet );
extern FILE *            Ntk_NetworkReadMvsisOut( Ntk_Network_t * pNet );
extern Ntk_Network_t *   Ntk_NetworkReadBackup( Ntk_Network_t * pNet );
extern int               Ntk_NetworkReadStep( Ntk_Network_t * pNet );
// network functionality APIs
extern Fnc_Manager_t *   Ntk_NetworkReadMan( Ntk_Network_t * pNet );
extern Vm_Manager_t *    Ntk_NetworkReadManVm( Ntk_Network_t * pNet );
extern Vmx_Manager_t *   Ntk_NetworkReadManVmx( Ntk_Network_t * pNet );
extern Mvc_Manager_t *   Ntk_NetworkReadManMvc( Ntk_Network_t * pNet );
extern Mvr_Manager_t *   Ntk_NetworkReadManMvr( Ntk_Network_t * pNet );
extern DdManager *       Ntk_NetworkReadManDdLoc( Ntk_Network_t * pNet );
extern DdManager *       Ntk_NetworkReadManDdGlo( Ntk_Network_t * pNet );
extern Ntk_Network_t *   Ntk_NetworkReadNetExdc( Ntk_Network_t * pNet );
// data-entry node APIs
extern void              Ntk_NodeSetName( Ntk_Node_t * pNode, char * pName );
extern void              Ntk_NodeSetSubtype( Ntk_Node_t * pNode, int Subtype );
extern void              Ntk_NodeSetValueNum( Ntk_Node_t * pNode, int nValues );
extern void              Ntk_NodeSetData( Ntk_Node_t * pNode, char * pData );
extern void              Ntk_NodeSetFuncCvr( Ntk_Node_t * pNode, Cvr_Cover_t * pCvr );
extern void              Ntk_NodeSetNetwork( Ntk_Node_t * pNode, Ntk_Network_t * pNet );
// data-entry network APIs
extern void              Ntk_NetworkSetNetExdc( Ntk_Network_t * pNet, Ntk_Network_t * pNetExdc );
extern void              Ntk_NetworkSetName( Ntk_Network_t * pNet, char * pName );
extern void              Ntk_NetworkSetSpec( Ntk_Network_t * pNet, char * pName );
extern void              Ntk_NetworkSetMvsis( Ntk_Network_t * pNet, Mv_Frame_t * pMvsis );
extern void              Ntk_NetworkSetBackup( Ntk_Network_t * pNet, Ntk_Network_t * pNetBackup );
extern void              Ntk_NetworkSetStep( Ntk_Network_t * pNet, int iStep );
// data-test node APIs
extern bool              Ntk_NodeIsCi( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsCo( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsCoDriver( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsCoFanin( Ntk_Node_t * pNode );
extern Ntk_Node_t *      Ntk_NodeHasCoName( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsInternal( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsConstant( Ntk_Node_t * pNode );
extern bool              Ntk_NodeBelongsToLatch( Ntk_Node_t * pNode );
// data-test network APIs
/*=== ntkCheck.c ====================================================*/
extern int               Ntk_NetworkCheck( Ntk_Network_t * pNet );
extern int               Ntk_NetworkCheckNode( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern int               Ntk_NetworkCheckLatch( Ntk_Network_t * pNet, Ntk_Latch_t * pLatch );
/*=== ntkDfs.c =======================================================*/
extern void              Ntk_NetworkDfs( Ntk_Network_t * pNet, int fFromOutputs );
extern void              Ntk_NetworkDfsNodes( Ntk_Network_t * pNet, Ntk_Node_t * ppNodes[], int nNodes, int fFromOutputs );
extern int               Ntk_NetworkLevelize( Ntk_Network_t * pNet, int fFromOutputs );
extern int               Ntk_NetworkLevelizeNodes( Ntk_Node_t * ppNodes[], int nNodes, int fFromOutputs );
extern int               Ntk_NetworkGetNumLevels( Ntk_Network_t * pNet );
extern bool              Ntk_NetworkIsAcyclic( Ntk_Network_t * pNet );
extern bool              Ntk_NetworkIsAcyclic1( Ntk_Network_t * pNet );
extern bool              Ntk_NetworkIsAcyclic2( Ntk_Network_t * pNet );
extern int               Ntk_NetworkComputeNodeSupport( Ntk_Network_t * pNet, Ntk_Node_t * pNodes[], int nNodes );
extern int               Ntk_NetworkInterleaveNodes( Ntk_Network_t * pNet, Ntk_Node_t * pNodes[], int nNodes );
extern int               Ntk_NetworkComputeNodeTfi( Ntk_Network_t * pNet, Ntk_Node_t * pNodes[], int nNodes, int Depth, bool fIncludeCis, bool fExistPath );
extern int               Ntk_NetworkComputeNodeTfo( Ntk_Network_t * pNet, Ntk_Node_t * pNodes[], int nNodes, int Depth, bool fIncludeCos, bool fExistPath ); 
extern bool              Ntk_NodeHasCoInTfo( Ntk_Node_t * pNode );
/*=== ntkFanio.c ===================================================*/
// node fanin/fanout APIs
extern Ntk_Node_t *      Ntk_NodeReadFaninNode( Ntk_Node_t * pNode, int i );
extern Ntk_Node_t *      Ntk_NodeReadFanoutNode( Ntk_Node_t * pNode, int i );
extern int               Ntk_NodeReadFaninIndex( Ntk_Node_t * pNode, Ntk_Node_t * pFanin );
extern int               Ntk_NodeReadFanoutIndex( Ntk_Node_t * pNode, Ntk_Node_t * pFanout );
extern int               Ntk_NodeReadFanins( Ntk_Node_t * pNode, Ntk_Node_t * pFanins[] );
extern int               Ntk_NodeReadFanouts( Ntk_Node_t * pNode, Ntk_Node_t * pFanouts[] );
extern void              Ntk_NodeReadFanioValues( Ntk_Node_t * pNode, int * pValues );
extern void              Ntk_NodeReduceFanins( Ntk_Node_t * pNode, int * pSupport );
extern Ntk_Pin_t *       Ntk_NodeAddFanin( Ntk_Node_t * pNode, Ntk_Node_t * pFanin );
extern Ntk_Pin_t *       Ntk_NodeAddFanout( Ntk_Node_t * pNode, Ntk_Node_t * pFanout );
extern void              Ntk_NodeAddFaninFanout( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern void              Ntk_NodeDeleteFaninFanout( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern void              Ntk_NodeTransferFanout( Ntk_Node_t * pNodeFrom, Ntk_Node_t * pNodeTo );
extern void              Ntk_NodePatchFanin( Ntk_Node_t * pNode, Ntk_Node_t * pFaninOld, Ntk_Node_t * pFaninNew );
extern Vm_VarMap_t *     Ntk_NodeAssignVm( Ntk_Node_t * pNode );
extern bool              Ntk_NodeSupportContain( Ntk_Node_t * pNode1, Ntk_Node_t * pNode2 );
/*=== ntkLatch.c ====================================================*/
extern Ntk_Latch_t *     Ntk_LatchCreate( Ntk_Network_t * pNet, Ntk_Node_t * pNode, char * pNameInput, char * pNameOutput );
extern Ntk_Latch_t *     Ntk_LatchDup( Ntk_Network_t * pNet, Ntk_Latch_t * pLatch );
extern void              Ntk_LatchDelete( Ntk_Latch_t * pLatch );
extern void              Ntk_LatchAdjustInput( Ntk_Network_t * pNet, Ntk_Latch_t * pLatch );
extern void              Ntk_NetworkAddLatch( Ntk_Network_t * pNet, Ntk_Latch_t * pLatch );
extern void              Ntk_NetworkDeleteLatch( Ntk_Latch_t * pLatch );
/*=== ntkNames.c ====================================================*/
extern char *            Ntk_NodeGetNamePrintable( Ntk_Node_t * pNode );
extern char *            Ntk_NodeGetNameLong( Ntk_Node_t * pNode );
extern char *            Ntk_NodeGetNameShort( Ntk_Node_t * pNode );
extern char *            Ntk_NodeGetNameUniqueLong( Ntk_Node_t * pNode );
extern char *            Ntk_NodeGetNameUniqueShort( Ntk_Node_t * pNode );
extern char *            Ntk_NodeAssignName( Ntk_Node_t * pNode, char * pName );
extern char *            Ntk_NodeRemoveName( Ntk_Node_t * pNode );
extern int               Ntk_NodeCompareByNameAndId( Ntk_Node_t ** ppN1, Ntk_Node_t ** ppN2 );
extern int               Ntk_NodeCompareByNameAndIdOrder( Ntk_Node_t ** ppN1, Ntk_Node_t ** ppN2 );
extern int               Ntk_NodeCompareByValueAndId( Ntk_Node_t ** ppN1, Ntk_Node_t ** ppN2 );
extern int               Ntk_NodeCompareByLevel( Ntk_Node_t ** ppN1, Ntk_Node_t ** ppN2 );
extern bool              Ntk_NetworkIsModeShort( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkFindOrAddNodeByName( Ntk_Network_t * pNet, char * pName, Ntk_NodeType_t Type );
extern Ntk_Node_t *      Ntk_NetworkFindNodeByName( Ntk_Network_t * pNet, char * pName );
extern Ntk_Node_t *      Ntk_NetworkFindNodeByNameWithMode( Ntk_Network_t * pNet, char * pName );
extern Ntk_Node_t *      Ntk_NetworkFindNodeByNameLong( Ntk_Network_t * pNet, char * pName );
extern Ntk_Node_t *      Ntk_NetworkFindNodeByNameShort( Ntk_Network_t * pNet, char * pName );
extern Ntk_Node_t *      Ntk_NetworkFindNodeById( Ntk_Network_t * pNet, int Id );
extern char *            Ntk_NetworkRegisterNewName( Ntk_Network_t * pNet, char * pName );
extern void              Ntk_NetworkRenameNodes( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkCollectNodeByName( Ntk_Network_t * pNet, char * pName, int fCollectCis );
extern void              Ntk_NetworkCollectIoNames( Ntk_Network_t * pNet, char *** ppsLeaves, char *** ppsRoots );
/*=== ntkNode.c =====================================================*/
extern Ntk_Node_t *      Ntk_NodeCreate( Ntk_Network_t * pNet, char * pName, Ntk_NodeType_t Type, int nValues );
extern Ntk_Node_t *      Ntk_NodeCreateConstant( Ntk_Network_t * pNet, int nValues, unsigned Pol );
extern Ntk_Node_t *      Ntk_NodeCreateOneInputNode( Ntk_Network_t * pNet, Ntk_Node_t * pFanin, int nValuesIn, int nValuesOut, unsigned Pols[] );
extern Ntk_Node_t *      Ntk_NodeCreateTwoInputBinary( Ntk_Network_t * pNet, Ntk_Node_t * ppFanins[], unsigned TruthTable );
extern Ntk_Node_t *      Ntk_NodeCreateDecoder( Ntk_Network_t * pNet, Ntk_Node_t * pNode1, Ntk_Node_t * pNode2 );
extern Ntk_Node_t *      Ntk_NodeCreateDecoderGeneral( Ntk_Node_t ** ppNodes, int nNodes, int * pValueAssign, int nTotalValues, int nOutputValues);
extern void              Ntk_NodeCreateEncoded( Ntk_Node_t * pNode, Ntk_Node_t ** ppNodes, int nNodes, int * pValueAssign, int nTotalValues, int nOutputValues);
extern void              Ntk_NodeCreateEncoders( Ntk_Network_t * pNet, Ntk_Node_t * pNode, int nValues1, int nValues2, Ntk_Node_t ** ppNode1, Ntk_Node_t ** ppNode2 );
extern Ntk_Node_t *      Ntk_NodeCreateCollector( Ntk_Network_t * pNet, Ntk_Node_t * ppFanins[], int nValues, int DefValue );
extern Ntk_Node_t *      Ntk_NodeCreateFromNetwork( Ntk_Network_t * pNet, char ** psCiNames );
extern Ntk_Node_t *      Ntk_NodeDup( Ntk_Network_t * pNetNew, Ntk_Node_t * pNode );
extern Ntk_Node_t *      Ntk_NodeClone( Ntk_Network_t * pNetNew, Ntk_Node_t * pNode );
extern void              Ntk_NodeReplace( Ntk_Node_t * pNode, Ntk_Node_t * pNodeNew );
extern void              Ntk_NodeDelete( Ntk_Node_t * pNode );
extern void              Ntk_NodeRecycle( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsTooLarge( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsBinaryBuffer( Ntk_Node_t * pNode );
extern bool              Ntk_NodesCanBeMerged( Ntk_Node_t * pNode1, Ntk_Node_t * pNode2 );
extern bool              Ntk_NodeDeterminize( Ntk_Node_t * pNode );
/*=== ntkNodeHeap.c =======================================================*/
extern Ntk_NodeHeap_t *  Ntk_NodeHeapStart();
extern void              Ntk_NodeHeapStop( Ntk_NodeHeap_t * p );
extern void              Ntk_NodeHeapPrint( FILE * pFile, Ntk_NodeHeap_t * p );
extern void              Ntk_NodeHeapCheck( Ntk_NodeHeap_t * p );
extern void              Ntk_NodeHeapCheckOne( Ntk_NodeHeap_t * p, Ntk_Node_t * pNode );
extern void              Ntk_NodeHeapInsert( Ntk_NodeHeap_t * p, Ntk_Node_t * pNode );  
extern void              Ntk_NodeHeapUpdate( Ntk_NodeHeap_t * p, Ntk_Node_t * pNode );  
extern void              Ntk_NodeHeapDelete( Ntk_NodeHeap_t * p, Ntk_Node_t * pNode );  
extern int               Ntk_NodeHeapReadMaxWeight( Ntk_NodeHeap_t * p );
extern int               Ntk_NodeHeapCountNodes( Ntk_NodeHeap_t * p, int WeightLimit );  
extern Ntk_Node_t *      Ntk_NodeHeapReadMax( Ntk_NodeHeap_t * p );  
extern Ntk_Node_t *      Ntk_NodeHeapGetMax( Ntk_NodeHeap_t * p );  
/*=== ntkSort.c ====================================================*/
extern int               Ntk_NetworkCompactNodeIds( Ntk_Network_t * pNet );
extern int               Ntk_NetworkOrderFanins( Ntk_Network_t * pNet );
extern int               Ntk_NodeOrderFanins( Ntk_Node_t * pNode );
extern int *             Ntk_NodeRelinkFanins( Ntk_Node_t * pNode );
extern void              Ntk_NetworkReassignIds( Ntk_Network_t * pNet );
extern void              Ntk_NodeCompactFanins( Ntk_Node_t * pNode, Ntk_Node_t * pFaninDup );
/*=== ntkSubnetwork.c ====================================================*/
extern Ntk_Network_t *   Ntk_SubnetworkExtract( Ntk_Network_t * pNet );
extern void              Ntk_SubnetworkInsert( Ntk_Network_t * pNet, Ntk_Network_t * pNetSub );
extern void              Ntk_SubnetworkWriteIntoFile( Ntk_Network_t * pNet, char * FileName, bool fBinary );
/*=== ntkUtils.c ====================================================*/
extern Ntk_Network_t *   Ntk_NetworkAlloc( Mv_Frame_t * pMvsis );
extern Ntk_Network_t *   Ntk_NetworkDup( Ntk_Network_t * pNet, Fnc_Manager_t * pMan );
extern void              Ntk_NetworkDelete( Ntk_Network_t * pNet );
extern void              Ntk_NetworkAddNode( Ntk_Network_t * pNet, Ntk_Node_t * pNode, bool fConnect ); 
extern Ntk_Node_t *      Ntk_NetworkAddNodeCo( Ntk_Network_t * pNet, Ntk_Node_t * pNode, bool fConnect );
extern void              Ntk_NetworkTransformNodeIntToCo( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern void              Ntk_NetworkTransformCiToCo( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern void              Ntk_NetworkTransformNodeIntToCi( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern void              Ntk_NetworkDeleteNode( Ntk_Network_t * pNet, Ntk_Node_t * pNode, bool fConnect, bool fDeleteNode ); 
extern bool              Ntk_NetworkIsBinary( Ntk_Network_t * pNet );
extern bool              Ntk_NetworkIsND( Ntk_Network_t * pNet );
extern Ntk_Node_t *      Ntk_NetworkReadCiNode( Ntk_Network_t * pNet, int i );
extern Ntk_Node_t *      Ntk_NetworkReadCoNode( Ntk_Network_t * pNet, int i );
extern int               Ntk_NetworkReadCiIndex( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
extern int               Ntk_NetworkReadCoIndex( Ntk_Network_t * pNet, Ntk_Node_t * pNode );
/*=== ntkTravId.c =========================================================*/
extern int               Ntk_NetworkReadTravId( Ntk_Network_t * pNet );
extern void              Ntk_NetworkIncrementTravId( Ntk_Network_t * pNet );
extern int               Ntk_NodeReadTravId( Ntk_Node_t * pNode );
extern void              Ntk_NodeSetTravId( Ntk_Node_t * pNode, int TravId );
extern void              Ntk_NodeSetTravIdCurrent( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsTravIdCurrent( Ntk_Node_t * pNode );
extern bool              Ntk_NodeIsTravIdPrevious( Ntk_Node_t * pNode );
extern void              Ntk_NetworkStartSpecial( Ntk_Network_t * pNet );
extern void              Ntk_NetworkStopSpecial( Ntk_Network_t * pNet );
extern void              Ntk_NetworkAddToSpecial( Ntk_Node_t * pNode );
extern void              Ntk_NetworkMoveSpecial( Ntk_Node_t * pNode );
extern void              Ntk_NetworkResetSpecial( Ntk_Network_t * pNet );
extern int               Ntk_NetworkCountSpecial( Ntk_Network_t * pNet );
extern void              Ntk_NetworkCreateSpecialFromArray( Ntk_Network_t * pNet, Ntk_Node_t * ppNodes[], int nNodes );
extern int               Ntk_NetworkCreateArrayFromSpecial( Ntk_Network_t * pNet, Ntk_Node_t * ppNodes[] );
extern void              Ntk_NetworkPrintSpecial( Ntk_Network_t * pNet );
extern void              Ntk_NetworkPrintArray( Ntk_Node_t ** ppNodes, int nNodes );

/*=== ntkMinBase.c ====================================================*/
extern bool              Ntk_NodeMakeMinimumBase( Ntk_Node_t * pNode );
/*=== ntkNodeCol.c ====================================================*/
extern bool              Ntk_NetworkCollapseNodes( Ntk_Node_t * pNode, Ntk_Node_t * pFanin );
extern Ntk_Node_t *      Ntk_NodeCollapse( Ntk_Node_t * pNode, Ntk_Node_t * pFanin );
extern Ntk_Node_t *      Ntk_NodeMakeCommonBase( Ntk_Node_t * pNode1, Ntk_Node_t * pNode2, int * pTransMapN1, int * pTransMapN2 );

////////////////////////////////////////////////////////////////////////
///                        ITERATORS                                 ///
////////////////////////////////////////////////////////////////////////
/*
    The safe iterators allow the user to delete the object
    that is currently being iterated through.
    Note the following important difference of the iterator
    through the nodes (Ntk_NetworkForEachNode) compared to SIS/VIS.
    The present iterator iterated only through the internal nodes,
    and not through all nodes (including PIs and POs) as before!
    This change led to certain simplification of the code. 
*/

// iterator through the primary inputs
#define Ntk_NetworkForEachCi( Net, Node )                          \
    for ( Node = Ntk_NetworkReadCiHead(Net);                       \
          Node;                                                    \
          Node = Ntk_NodeReadNext(Node) )
#define Ntk_NetworkForEachCiSafe( Net, Node, Node2 )               \
    for ( Node = Ntk_NetworkReadCiHead(Net),                       \
          Node2 = (Node? Ntk_NodeReadNext(Node): NULL);            \
          Node;                                                    \
          Node = Node2,                                            \
          Node2 = (Node? Ntk_NodeReadNext(Node): NULL) )

// iterator through the primary outputs
#define Ntk_NetworkForEachCo( Net, Node )                          \
    for ( Node = Ntk_NetworkReadCoHead(Net);                       \
          Node;                                                    \
          Node = Ntk_NodeReadNext(Node) )
#define Ntk_NetworkForEachCoSafe( Net, Node, Node2 )               \
    for ( Node = Ntk_NetworkReadCoHead(Net),                       \
          Node2 = (Node? Ntk_NodeReadNext(Node): NULL);            \
          Node;                                                    \
          Node = Node2,                                            \
          Node2 = (Node? Ntk_NodeReadNext(Node): NULL) )

// iterator through the primary output drivers
#define Ntk_NetworkForEachCoDriver( Net, NodeCo, Driver )          \
    for ( NodeCo = Ntk_NetworkReadCoHead(Net),                     \
          Driver = NodeCo? Ntk_NodeReadFaninNode(NodeCo,0): NULL;  \
          NodeCo;                                                  \
          NodeCo = Ntk_NodeReadNext(NodeCo),                       \
          Driver = NodeCo? Ntk_NodeReadFaninNode(NodeCo,0): NULL )

// iterator through the internal nodes
#define Ntk_NetworkForEachNode( Net, Node )                        \
    for ( Node = Ntk_NetworkReadNodeHead(Net);                     \
          Node;                                                    \
          Node = Ntk_NodeReadNext(Node) )
#define Ntk_NetworkForEachNodeSafe( Net, Node, Node2 )             \
    for ( Node = Ntk_NetworkReadNodeHead(Net),                     \
          Node2 = (Node? Ntk_NodeReadNext(Node): NULL);            \
          Node;                                                    \
          Node = Node2,                                            \
          Node2 = (Node? Ntk_NodeReadNext(Node): NULL) )

// specialized iterator through the ordered nodes
#define Ntk_NetworkForEachNodeSpecial( Net, Node )                 \
    for ( Node = Ntk_NetworkReadOrder(Net);                        \
          Node;                                                    \
          Node = Ntk_NodeReadOrder(Node) )
#define Ntk_NetworkForEachNodeSpecialSafe( Net, Node, Node2 )      \
    for ( Node = Ntk_NetworkReadOrder(Net),                        \
          Node2 = (Node? Ntk_NodeReadOrder(Node): NULL);                      \
          Node;                                                    \
          Node = Node2,                                            \
          Node2 = (Node? Ntk_NodeReadOrder(Node): NULL) )


// specialized iterator through the nodes in the levelized structure
#define Ntk_NetworkForEachNodeSpecialByLevel( Net, Level, Node )   \
    for ( Node = Ntk_NetworkReadOrderByLevel(Net, Level);          \
          Node;                                                    \
          Node = Ntk_NodeReadOrder(Node) )
#define Ntk_NetworkForEachNodeSpecialByLevelSafe( Net, Level, Node, Node2 )\
    for ( Node = Ntk_NetworkReadOrderByLevel(Net, Level),          \
          Node2 = (Node? Ntk_NodeReadOrder(Node): NULL);                      \
          Node;                                                    \
          Node = Node2,                                            \
          Node2 = (Node? Ntk_NodeReadOrder(Node): NULL) )



// iterator through latches
#define Ntk_NetworkForEachLatch( Net, Latch )                      \
    for ( Latch = Ntk_NetworkReadLatchHead(Net);                   \
          Latch;                                                   \
          Latch = Ntk_LatchReadNext(Latch) )
#define Ntk_NetworkForEachLatchSafe( Net, Latch, Latch2 )          \
    for ( Latch = Ntk_NetworkReadLatchHead(Net),                   \
          Latch2 = (Latch? Ntk_LatchReadNext(Latch): NULL);        \
          Latch;                                                   \
          Latch = Latch2,                                          \
          Latch2 = (Latch? Ntk_LatchReadNext(Latch): NULL) )


// iterator through the fanins of the node
#define Ntk_NodeForEachFanin( Node, Pin, Fanin )                   \
    for ( Pin = Ntk_NodeReadFaninPinHead(Node);                    \
          Pin && (((unsigned)(Fanin = Ntk_PinReadNode(Pin)))>=0);  \
          Pin = Ntk_PinReadNext(Pin) )
#define Ntk_NodeForEachFaninSafe( Node, Pin, Pin2, Fanin )         \
    for ( Pin = Ntk_NodeReadFaninPinHead(Node),                    \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL);                \
          Pin && (((unsigned)(Fanin = Ntk_PinReadNode(Pin)))>=0);  \
          Pin = Pin2,                                              \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL) )

// iterator through the fanouts of the node
#define Ntk_NodeForEachFanout( Node, Pin, Fanout )                 \
    for ( Pin = Ntk_NodeReadFanoutPinHead(Node);                   \
          Pin && (((unsigned)(Fanout = Ntk_PinReadNode(Pin)))>=0); \
          Pin = Ntk_PinReadNext(Pin) )
#define Ntk_NodeForEachFanoutSafe( Node, Pin, Pin2, Fanout )       \
    for ( Pin = Ntk_NodeReadFanoutPinHead(Node),                   \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL);                \
          Pin && (((unsigned)(Fanout = Ntk_PinReadNode(Pin)))>=0); \
          Pin = Pin2,                                              \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL) )


// iterator through the fanins of the node with fanin index
#define Ntk_NodeForEachFaninWithIndex( Node, Pin, Fanin, i )       \
    for ( Pin = Ntk_NodeReadFaninPinHead(Node), i = 0;             \
          Pin && (((unsigned)(Fanin = Ntk_PinReadNode(Pin)))>=0);  \
          Pin = Ntk_PinReadNext(Pin), i++ )
#define Ntk_NodeForEachFaninWithIndexSafe( Node, Pin, Pin2, Fanin, i )\
    for ( Pin = Ntk_NodeReadFaninPinHead(Node),                    \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL), i = 0;         \
          Pin && (((unsigned)(Fanin = Ntk_PinReadNode(Pin)))>=0);  \
          Pin = Pin2,                                              \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL), i++ )

// iterator through the fanouts of the node with fanout index
#define Ntk_NodeForEachFanoutWithIndex( Node, Pin, Fanout, i )     \
    for ( Pin = Ntk_NodeReadFanoutPinHead(Node), i = 0;            \
          Pin && (((unsigned)(Fanout = Ntk_PinReadNode(Pin)))>=0); \
          Pin = Ntk_PinReadNext(Pin), i++ )
#define Ntk_NodeForEachFanoutWithIndexSafe( Node, Pin, Pin2, Fanout, i )\
    for ( Pin = Ntk_NodeReadFanoutPinHead(Node),                   \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL), i = 0;         \
          Pin && (((unsigned)(Fanout = Ntk_PinReadNode(Pin)))>=0); \
          Pin = Pin2,                                              \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL), i++ )


// iterator through the fanouts of the node with fanout's fanin index
#define Ntk_NodeForEachFanoutWithFaninIndex( Node, Pin, Fanout, FaninIndex )\
    for ( Pin = Ntk_NodeReadFanoutPinHead(Node);                   \
          Pin && (((unsigned)(Fanout = Ntk_PinReadNode(Pin)))>=0) &&\
          (((unsigned)(FaninIndex = Ntk_NodeReadFaninIndex(Fanout,Node)))>=0);\
          Pin = Ntk_PinReadNext(Pin), i++ )
#define Ntk_NodeForEachFanoutWithFaninIndexSafe( Node, Pin, Pin2, Fanout, FaninIndex )\
    for ( Pin = Ntk_NodeReadFanoutPinHead(Node),                   \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL), i = 0;         \
          Pin && (((unsigned)(Fanout = Ntk_PinReadNode(Pin)))>=0) &&\
          (((unsigned)(FaninIndex = Ntk_NodeReadFaninIndex(Fanout,Node)))>=0);\
          Pin = Pin2,                                              \
          Pin2 = (Pin? Ntk_PinReadNext(Pin): NULL), i++ )

////////////////////////////////////////////////////////////////////////
///                       END OF FILE                                ///
////////////////////////////////////////////////////////////////////////

#endif

